/***********************************
This is a our graphics core library, for all our displays. 
We'll be adapting all the
existing libaries to use this core to make updating, support 
and upgrading easier!

Adafruit invests time and resources providing this open source code, 
please support Adafruit and open-source hardware by purchasing 
products from Adafruit!

Written by Limor Fried/Ladyada  for Adafruit Industries.  
BSD license, check license.txt for more information
All text above must be included in any redistribution
****************************************/


//CS 	= C0
//CD 	= C1
//WR 	= C2
//RD 	= C3
//RST	= C4


#include "Adafruit_GFX.h"
#include "glcdfont.c"
#include <avr/pgmspace.h>
#include <stdlib.h>
#include <util/delay.h>
#include "uart.h"
#include "TouchScreen.h"

 #define DATAPORT1 PORTD
 #define DATAPIN1  PIND
 #define DATADDR1  DDRD

 #define DATAPORT2 PORTB
 #define DATAPIN2  PINB
 #define DATADDR2  DDRB

 #define DATA1_MASK 0xFC  // top 6 bits
 #define DATA2_MASK 0x03  // bottom 2 bits


//Function defintions from GFX

void constructor(uint16_t w, uint16_t h) {
  _width = WIDTH = w;
  _height = HEIGHT = h;

  rotation = 0;    
  cursor_y = cursor_x = 0;
  textsize = 1;
  textcolor = 0xFFFF;
  textbgcolor = 0x0000;
}


// draw a circle outline
void drawCircle(uint16_t x0, uint16_t y0, uint16_t r, 
			      uint16_t color) {
  int16_t f = 1 - r;
  int16_t ddF_x = 1;
  int16_t ddF_y = -2 * r;
  int16_t x = 0;
  int16_t y = r;

  drawPixel(x0, y0+r, color);
  drawPixel(x0, y0-r, color);
  drawPixel(x0+r, y0, color);
  drawPixel(x0-r, y0, color);

  while (x<y) {
    if (f >= 0) {
      y--;
      ddF_y += 2;
      f += ddF_y;
    }
    x++;
    ddF_x += 2;
    f += ddF_x;
  
    drawPixel(x0 + x, y0 + y, color);
    drawPixel(x0 - x, y0 + y, color);
    drawPixel(x0 + x, y0 - y, color);
    drawPixel(x0 - x, y0 - y, color);
    drawPixel(x0 + y, y0 + x, color);
    drawPixel(x0 - y, y0 + x, color);
    drawPixel(x0 + y, y0 - x, color);
    drawPixel(x0 - y, y0 - x, color);
    
  }
}
//Helper function for drawing a circle
void drawCircleHelper(uint16_t x0, uint16_t y0,
				       uint16_t r, uint8_t cornername, uint16_t color) {
  int16_t f     = 1 - r;
  int16_t ddF_x = 1;
  int16_t ddF_y = -2 * r;
  int16_t x     = 0;
  int16_t y     = r;

  while (x<y) {
    if (f >= 0) {
      y--;
      ddF_y += 2;
      f     += ddF_y;
    }
    x++;
    ddF_x += 2;
    f     += ddF_x;
    if (cornername & 0x4) {
      drawPixel(x0 + x, y0 + y, color);
      drawPixel(x0 + y, y0 + x, color);
    } 
    if (cornername & 0x2) {
      drawPixel(x0 + x, y0 - y, color);
      drawPixel(x0 + y, y0 - x, color);
    }
    if (cornername & 0x8) {
      drawPixel(x0 - y, y0 + x, color);
      drawPixel(x0 - x, y0 + y, color);
    }
    if (cornername & 0x1) {
      drawPixel(x0 - y, y0 - x, color);
      drawPixel(x0 - x, y0 - y, color);
    }
  }
}
//Draws a circle
void fillCircle(uint16_t x0, uint16_t y0, uint16_t r, 
			      uint16_t color) {
  drawFastVLine(x0, y0-r, 2*r+1, color);
  fillCircleHelper(x0, y0, r, 3, 0, color);
}
//Draws the left half of a circle
void fillLeftHalfCircle(uint16_t x0, uint16_t y0, uint16_t r, 
			      uint16_t color) {
  drawFastVLine(x0, y0-r, 2*r+1, color);
  fillCircleHelper(x0, y0, r, 2, 0, color);
}
//Draws the right half of a circle
void fillRightHalfCircle(uint16_t x0, uint16_t y0, uint16_t r, 
			      uint16_t color) {
  drawFastVLine(x0, y0-r, 2*r+1, color);
  fillCircleHelper(x0, y0, r, 1, 0, color);
}
//Draws the top half of a circle
void fillTopHalfCircle(uint16_t x0, uint16_t y0, uint16_t r, 
			      uint16_t color)
{
  int16_t f     = 1 - r;
  int16_t ddF_x = 1;
  int16_t ddF_y = -2 * r;
  int16_t x     = 0;
  int16_t y     = r;

  drawFastVLine(x0, y0-r, 2*r+1, color);
  while (x<y) {
    if (f >= 0) {
      y--;
      ddF_y += 2;
      f     += ddF_y;
    }
    x++;
    ddF_x += 2;
    f     += ddF_x;

      drawFastVLine(x0+x, y0-y, y+1, color);
      drawFastVLine(x0+y, y0-x, x+1, color);
      drawFastVLine(x0-x, y0-y, y+1, color);
      drawFastVLine(x0-y, y0-x, x+1, color);
  }
}
//Draws the bottom half of a circle
void fillBottomHalfCircle(uint16_t x0, uint16_t y0, uint16_t r, 
			      uint16_t color)
{
  int16_t f     = 1 - r;
  int16_t ddF_x = 1;
  int16_t ddF_y = -2 * r;
  int16_t x     = 0;
  int16_t y     = r;

  drawFastVLine(x0, y0-r, 2*r+1, color);
  while (x<y) {
    if (f >= 0) {
      y--;
      ddF_y += 2;
      f     += ddF_y;
    }
    x++;
    ddF_x += 2;
    f     += ddF_x;

      drawFastVLine(x0+x, y0, 2*y+1, color);
      drawFastVLine(x0+y, y0, 2*x+1, color);
      drawFastVLine(x0-x, y0, 2*y+1, color);
      drawFastVLine(x0-y, y0, 2*x+1, color);
  }
}

// used to do circles and roundrects!
void fillCircleHelper(uint16_t x0, uint16_t y0, uint16_t r,
				    uint8_t cornername, uint16_t delta, uint16_t color) {

  int16_t f     = 1 - r;
  int16_t ddF_x = 1;
  int16_t ddF_y = -2 * r;
  int16_t x     = 0;
  int16_t y     = r;

  while (x<y) {
    if (f >= 0) {
      y--;
      ddF_y += 2;
      f     += ddF_y;
    }
    x++;
    ddF_x += 2;
    f     += ddF_x;

    if (cornername & 0x1) {
      drawFastVLine(x0+x, y0-y, 2*y+1+delta, color);
      drawFastVLine(x0+y, y0-x, 2*x+1+delta, color);
    }
    if (cornername & 0x2) {
      drawFastVLine(x0-x, y0-y, 2*y+1+delta, color);
      drawFastVLine(x0-y, y0-x, 2*x+1+delta, color);
    }
  }
}

// bresenham's algorithm - thx wikpedia
void drawLine(int16_t x0, int16_t y0, 
			    int16_t x1, int16_t y1, 
			    uint16_t color) {
  int16_t steep = abs(y1 - y0) > abs(x1 - x0);
  if (steep) {
    swap(x0, y0);
    swap(x1, y1);
  }

  if (x0 > x1) {
    swap(x0, x1);
    swap(y0, y1);
  }

  int16_t dx, dy;
  dx = x1 - x0;
  dy = abs(y1 - y0);

  int16_t err = dx / 2;
  int16_t ystep;

  if (y0 < y1) {
    ystep = 1;
  } else {
    ystep = -1;
  }

  for (; x0<=x1; x0++) {
    if (steep) {
      drawPixel(y0, x0, color);
    } else {
      drawPixel(x0, y0, color);
    }
    err -= dy;
    if (err < 0) {
      y0 += ystep;
      err += dx;
    }
  }
}


// draw a rectangle
void drawRect(uint16_t x, uint16_t y, 
			    uint16_t w, uint16_t h, 
			    uint16_t color) {
  drawFastHLine(x, y, w, color);
  drawFastHLine(x, y+h-1, w, color);
  drawFastVLine(x, y, h, color);
  drawFastVLine(x+w-1, y, h, color);
}

void drawFastVLine(uint16_t x, uint16_t y, 
				 uint16_t h, uint16_t color) {
  // stupidest version - update in subclasses if desired!
  drawLine(x, y, x, y+h-1, color);
}


void drawFastHLine(uint16_t x, uint16_t y, 
				 uint16_t w, uint16_t color) {
  // stupidest version - update in subclasses if desired!
  drawLine(x, y, x+w-1, y, color);
}

void fillRect(uint16_t x, uint16_t y, uint16_t w, uint16_t h, 
			    uint16_t color) {
  // stupidest version - update in subclasses if desired!
  for (uint16_t i=x; i<x+w; i++) {
    drawFastVLine(i, y, h, color); 
  }
}


/*void fillScreen(uint16_t color) {
  fillRect(0, 0, _width, _height, color);
}*/

// draw a rounded rectangle!
void drawRoundRect(uint16_t x, uint16_t y, uint16_t w,
uint16_t h, uint16_t r, uint16_t color) {
  // smarter version
  drawFastHLine(x+r  , y    , w-2*r, color); // Top
  drawFastHLine(x+r  , y+h-1, w-2*r, color); // Bottom
  drawFastVLine(  x    , y+r  , h-2*r, color); // Left
  drawFastVLine(  x+w-1, y+r  , h-2*r, color); // Right
  // draw four corners
  drawCircleHelper(x+r    , y+r    , r, 1, color);
  drawCircleHelper(x+w-r-1, y+r    , r, 2, color);
  drawCircleHelper(x+w-r-1, y+h-r-1, r, 4, color);
  drawCircleHelper(x+r    , y+h-r-1, r, 8, color);
}

// fill a rounded rectangle!
void fillRoundRect(uint16_t x, uint16_t y, uint16_t w,
				 uint16_t h, uint16_t r, uint16_t color) {
  // smarter version
  fillRect(x+r, y, w-2*r, h, color);

  // draw four corners
  fillCircleHelper(x+w-r-1, y+r, r, 1, h-2*r-1, color);
  fillCircleHelper(x+r    , y+r, r, 2, h-2*r-1, color);
}

// draw a triangle!
void drawTriangle(uint16_t x0, uint16_t y0,
				uint16_t x1, uint16_t y1, 
				uint16_t x2, uint16_t y2, uint16_t color) {
  drawLine(x0, y0, x1, y1, color);
  drawLine(x1, y1, x2, y2, color);
  drawLine(x2, y2, x0, y0, color);
}

// fill a triangle!
void fillTriangle ( uint16_t x0, uint16_t y0,
				     uint16_t x1, uint16_t y1, 
				     uint16_t x2, uint16_t y2, uint16_t color) {

  int16_t a, b, y, last;

  // Sort coordinates by Y order (y2 >= y1 >= y0)
  if (y0 > y1) {
    swap(y0, y1); swap(x0, x1);
  }
  if (y1 > y2) {
    swap(y2, y1); swap(x2, x1);
  }
  if (y0 > y1) {
    swap(y0, y1); swap(x0, x1);
  }

  if(y0 == y2) { // Handle awkward all-on-same-line case as its own thing
    a = b = x0;
    if(x1 < a)      a = x1;
    else if(x1 > b) b = x1;
    if(x2 < a)      a = x2;
    else if(x2 > b) b = x2;
    drawFastHLine(a, y0, b-a+1, color);
    return;
  }

  int16_t
    dx01 = x1 - x0,
    dy01 = y1 - y0,
    dx02 = x2 - x0,
    dy02 = y2 - y0,
    dx12 = x2 - x1,
    dy12 = y2 - y1,
    sa   = 0,
    sb   = 0;

  // For upper part of triangle, find scanline crossings for segments
  // 0-1 and 0-2.  If y1=y2 (flat-bottomed triangle), the scanline y1
  // is included here (and second loop will be skipped, avoiding a /0
  // error there), otherwise scanline y1 is skipped here and handled
  // in the second loop...which also avoids a /0 error here if y0=y1
  // (flat-topped triangle).
  if(y1 == y2) last = y1;   // Include y1 scanline
  else         last = y1-1; // Skip it

  for(y=y0; y<=last; y++) {
    a   = x0 + sa / dy01;
    b   = x0 + sb / dy02;
    sa += dx01;
    sb += dx02;
    /* longhand:
    a = x0 + (x1 - x0) * (y - y0) / (y1 - y0);
    b = x0 + (x2 - x0) * (y - y0) / (y2 - y0);
    */
    if(a > b) swap(a,b);
    drawFastHLine(a, y, b-a+1, color);
  }

  // For lower part of triangle, find scanline crossings for segments
  // 0-2 and 1-2.  This loop is skipped if y1=y2.
  sa = dx12 * (y - y1);
  sb = dx02 * (y - y0);
  for(; y<=y2; y++) {
    a   = x1 + sa / dy12;
    b   = x0 + sb / dy02;
    sa += dx12;
    sb += dx02;
    /* longhand:
    a = x1 + (x2 - x1) * (y - y1) / (y2 - y1);
    b = x0 + (x2 - x0) * (y - y0) / (y2 - y0);
    */
    if(a > b) swap(a,b);
    drawFastHLine(a, y, b-a+1, color);
  }
}
//Draws a bitmap from program memory.
//Can be used to draw pictures read from an SD card
void drawBitmap(uint16_t x, uint16_t y, 
			      const uint8_t *bitmap, uint16_t w, uint16_t h,
			      uint16_t color) {
  for (uint16_t j=0; j<h; j++) {
    for (uint16_t i=0; i<w; i++ ) {
      if (pgm_read_byte(bitmap + i + (j/8)*w) & _BV(j%8)) {
	drawPixel(x+i, y+j, color);
      }
    }
  }
}


//Writes a character c
void write(uint8_t c) {
  if (c == '\n') {
    cursor_y += textsize*8;
    cursor_x = 0;
  } else if (c == '\r') {
    // skip em
  } else {
    drawChar(cursor_x, cursor_y, c, textcolor, textbgcolor, textsize);
    cursor_x += textsize*6;
    if (cursor_x > (width() - textsize*6)) {
      cursor_y += textsize*8;
      cursor_x = 0;
    }
  }
}

// draw a character
void drawChar(uint16_t x, uint16_t y, char c,
			    uint16_t color, uint16_t bg, uint8_t size) {
  for (uint8_t i=0; i<6; i++ ) {
    uint8_t line;
    if (i == 5) 
      line = 0x0;
    else 
      line = pgm_read_byte(font+(c*5)+i);
    for (uint8_t j = 0; j<8; j++) {
      if (line & 0x1) {
        if (size == 1) // default size
          drawPixel(x+i, y+j, color);
        else {  // big size
          fillRect(x+(i*size), y+(j*size), size, size, color);
        } 
      } else if (bg != color) {
        if (size == 1) // default size
          drawPixel(x+i, y+j, bg);
        else {  // big size
          fillRect(x+i*size, y+j*size, size, size, bg);
        } 	
      }
      line >>= 1;
    }
  }
}
//Sets text cursor positon
void setCursor(uint16_t x, uint16_t y) {
  cursor_x = x;
  cursor_y = y;
}

//Sets the text size
void setTextSize(uint8_t s) {
  textsize = (s > 0) ? s : 1;
}

//Sets the text color
void setTextColor(uint16_t c) {
  textcolor = c;
  textbgcolor = c; 
  // for 'transparent' background, we'll set the bg 
  // to the same as fg instead of using a flag
}
//Sets the text color
 void setTextColor2(uint16_t c, uint16_t b) {
   textcolor = c;
   textbgcolor = b; 
 }
//Returns the rotation
uint8_t getRotation(void) {
  rotation %= 4;
  return rotation;
}

void setRotation(uint8_t x) {
  x %= 4;  // cant be higher than 3
  rotation = x;
  switch (x) {
  case 0:
  case 2:
    _width = WIDTH;
    _height = HEIGHT;
    break;
  case 1:
  case 3:
    _width = HEIGHT;
    _height = WIDTH;
    break;
 }
}



// return the size of the display which depends on the rotation!
//Returns width in pixels
 uint16_t width(void) { 
   return _width; 
}
 //Returns height in pixels
 uint16_t height(void) { 
   return _height; 
 }


 //Functions from TFT


void goTo(int x, int y) {
  if (driver == 0x9325 || driver == 0x9328) {
    writeRegister16(0x0020, x);     // GRAM Address Set (Horizontal Address) (R20h)
    writeRegister16(0x0021, y);     // GRAM Address Set (Vertical Address) (R21h)
    writeCommand(0x0022);            // Write Data to GRAM (R22h)
	//fprintf(stdout,"GoTo Command used the correct driver!\n\r");
  } 
  if (driver == 0x7575) {
    writeRegister8(HX8347G_COLADDRSTART2, x>>8);
    writeRegister8(HX8347G_COLADDRSTART1, x);
    writeRegister8(HX8347G_ROWADDRSTART2, y>>8);
    writeRegister8(HX8347G_ROWADDRSTART1, y);

    writeRegister8(HX8347G_COLADDREND2, 0);
    writeRegister8(HX8347G_COLADDREND1, TFTWIDTH-1);
    writeRegister8(HX8347G_ROWADDREND2, (TFTHEIGHT-1)>>8);
    writeRegister8(HX8347G_ROWADDREND1, (TFTHEIGHT-1));
    writeCommand(0x0022);            // Write Data to GRAM (R22h)
  }
}

//Converts a 24-bit RGB color to a 16-bit RGB color
uint16_t Color565(uint8_t r, uint8_t g, uint8_t b) {
  uint16_t c;
  c = r >> 3;
  c <<= 6;
  c |= g >> 2;
  c <<= 5;
  c |= b >> 3;

  return c;
}
//Draws a line at x,y of the specified length
void drawFastLine(uint16_t x, uint16_t y, uint16_t length, 
			  uint16_t color, uint8_t rotflag)
{
  uint16_t newentrymod=0;
  
  switch (rotation) {
  case 0:
    if (rotflag)
      newentrymod = 0x1028;   // we want a 'vertical line'
    else 
      newentrymod = 0x1030;   // we want a 'horizontal line'
    break;
  case 1:
    swap(x, y);
    // first up fix the X
    x = TFTWIDTH - x - 1;
    if (rotflag)
      newentrymod = 0x1000;   // we want a 'vertical line'
    else 
      newentrymod = 0x1028;   // we want a 'horizontal line'
    break;
  case 2:
    x = TFTWIDTH - x - 1;
    y = TFTHEIGHT - y - 1;
    if (rotflag)
      newentrymod = 0x1008;   // we want a 'vertical line'
    else 
      newentrymod = 0x1020;   // we want a 'horizontal line'
    break;
  case 3:
    swap(x,y);
    y = TFTHEIGHT - y - 1;
    if (rotflag)
      newentrymod = 0x1030;   // we want a 'vertical line'
    else 
      newentrymod = 0x1008;   // we want a 'horizontal line'
    break;
  }
  
  if ((driver == 0x9325) || (driver == 0x9328)) {

    writeRegister16(ILI932X_ENTRY_MOD, newentrymod);
    
    writeRegister16(ILI932X_GRAM_HOR_AD, x); // GRAM Address Set (Horizontal Address) (R20h)
    writeRegister16(ILI932X_GRAM_VER_AD, y); // GRAM Address Set (Vertical Address) (R21h)
    writeCommand(ILI932X_RW_GRAM);  // Write Data to GRAM (R22h)
  }
  if (driver == 0x7575) {
    writeRegister8(HX8347G_COLADDRSTART2, x>>8);
    writeRegister8(HX8347G_COLADDRSTART1, x);
    writeRegister8(HX8347G_ROWADDRSTART2, y>>8);
    writeRegister8(HX8347G_ROWADDRSTART1, y);

    uint16_t endx = x, endy = y;
    if (rotflag) {
      endy += length;
    } else {
      endx += length;
    }
    
    writeRegister8(HX8347G_COLADDREND2, endx>>8);
    writeRegister8(HX8347G_COLADDREND1, endx);
    writeRegister8(HX8347G_ROWADDREND2, endy>>8);
    writeRegister8(HX8347G_ROWADDREND1, endy);
    writeCommand(0x0022);            // Write Data to GRAM (R22h)


  }
  //Should be done
  /*portOutputRegister(csport) &= ~cspin;  //digitalWrite(_cs, LOW);
  portOutputRegister(cdport) |= cdpin;   //digitalWrite(_cd, HIGH);
  portOutputRegister(rdport) |= rdpin;   //digitalWrite(_rd, HIGH);
  portOutputRegister(wrport) |= wrpin;   //digitalWrite(_wr, HIGH); */
  PORTC &= ~(1<<PINC0);
  PORTC |= (1<<PINC1) | (1<<PINC2) | (1<<PINC3);

  setWriteDir();
  while (length--) {
    writeData_unsafe(color); 
  }

  // set back to default
  //*portOutputRegister(csport) |= cspin;    //digitalWrite(_cs, HIGH);
  //Should be done
  PORTC |= (1<<PINC0);
  writeRegister16(ILI932X_ENTRY_MOD, 0x1030);

  if (driver == 0x7575) {
    goTo(0,0);
  }
}


//Fills the screen with the specified color
void fillScreen(uint16_t color) {
  goTo(0,0);

  uint32_t i;
  
  i = 320;
  i *= 240;
  
  //*portOutputRegister(csport) &= ~cspin;    //digitalWrite(_cs, LOW);
  //*portOutputRegister(cdport) |= cdpin;     //digitalWrite(_cd, HIGH);
  //*portOutputRegister(rdport) |= rdpin;     //digitalWrite(_rd, HIGH);
  //*portOutputRegister(wrport) |= wrpin;     //digitalWrite(_wr, HIGH);
  PORTC &= ~(1<<PINC0);
  PORTC |= (1<<PINC1) | (1<<PINC2) | (1<<PINC3);

  setWriteDir();
  while (i--) {
    writeData(color); 
  }

  //*portOutputRegister(csport) |= cspin;    //digitalWrite(_cs, HIGH);
  PORTC |= (1<<PINC0);
  //fprintf(stdout,"Done writting colors!\n\r");
}
//Changes the color of the specified pixel
void drawPixel(uint16_t x, uint16_t y, uint16_t color)
{
  // check rotation, move pixel around if necessary
  switch (rotation) {
  case 1:
    swap(x, y);
    x = TFTWIDTH - x - 1;
    break;
  case 2:
    x = TFTWIDTH - x - 1;
    y = TFTHEIGHT - y - 1;
    break;
  case 3:
    swap(x, y);
    y = TFTHEIGHT - y - 1;
    break;
  }
    
  if ((x >= TFTWIDTH) || (y >= TFTHEIGHT)) return;

  if (driver == 0x9328 || driver == 0x9325) {
    writeRegister16(ILI932X_GRAM_HOR_AD, x); // GRAM Address Set (Horizontal Address)
    writeRegister16(ILI932X_GRAM_VER_AD, y); // GRAM Address Set (Vertical Address)
    writeCommand(ILI932X_RW_GRAM);  // Write Data to GRAM
  } else if (driver == 0x7575) {
    writeRegister8(HX8347G_COLADDRSTART2, x >> 8);
    writeRegister8(HX8347G_COLADDRSTART1, x & 0xFF);
    writeRegister8(HX8347G_ROWADDRSTART2, y >> 8);
    writeRegister8(HX8347G_ROWADDRSTART1, y & 0xFF);
    writeCommand8(0x0022);            // Write Data to GRAM (R22h)
  }
  writeData(color);
}


static const uint16_t HX8347G_regValues[] PROGMEM = {
  0x2E, 0x89,
  0x29, 0x8F,
  0x2B, 0x02,
  0xE2, 0x00,
  0xE4, 0x01,
  0xE5, 0x10,
  0xE6, 0x01,
  0xE7, 0x10,
  0xE8, 0x70,
  0xF2, 0x00,
  0xEA, 0x00,
  0xEB, 0x20,
  0xEC, 0x3C,
  0xED, 0xC8,
  0xE9, 0x38,
  0xF1, 0x01,

  // skip gamma, do later

  0x1B, 0x1A,
  0x1A, 0x02,
  0x24, 0x61,
  0x25, 0x5C,
  
  0x18, 0x36,
  0x19, 0x01,
  0x1F, 0x88,
  TFTLCD_DELAYCMD, 5, // delay 5 ms
  0x1F, 0x80,
  TFTLCD_DELAYCMD, 5,
  0x1F, 0x90,
  TFTLCD_DELAYCMD, 5,
  0x1F, 0xD4,
  TFTLCD_DELAYCMD, 5,
  0x17, 0x05,

  0x36, 0x09,
  0x28, 0x38,
  TFTLCD_DELAYCMD, 40,
  0x28, 0x3C,

  0x02, 0x00,
  0x03, 0x00,
  0x04, 0x00,
  0x05, 0xEF,
  0x06, 0x00,
  0x07, 0x00,
  0x08, 0x01,
  0x09, 0x3F,
};

static const uint16_t ILI932x_regValues[] PROGMEM = {
  ILI932X_START_OSC, 0x0001,     // start oscillator

  TFTLCD_DELAYCMD, 50,          // this will make a delay of 50 milliseconds

  ILI932X_DRIV_OUT_CTRL, 0x0100, 
  ILI932X_DRIV_WAV_CTRL, 0x0700,
  ILI932X_ENTRY_MOD, 0x1030,
  ILI932X_RESIZE_CTRL, 0x0000,
  ILI932X_DISP_CTRL2, 0x0202,
  ILI932X_DISP_CTRL3, 0x0000,
  ILI932X_DISP_CTRL4, 0x0000,
  ILI932X_RGB_DISP_IF_CTRL1, 0x0,
  ILI932X_FRM_MARKER_POS, 0x0,
  ILI932X_RGB_DISP_IF_CTRL2, 0x0,
  
  ILI932X_POW_CTRL1, 0x0000,
  ILI932X_POW_CTRL2, 0x0007,
  ILI932X_POW_CTRL3, 0x0000,
  ILI932X_POW_CTRL4, 0x0000,

  TFTLCD_DELAYCMD, 200,  
  
  ILI932X_POW_CTRL1, 0x1690,
  ILI932X_POW_CTRL2, 0x0227,

  TFTLCD_DELAYCMD, 50,  

  ILI932X_POW_CTRL3, 0x001A,

  TFTLCD_DELAYCMD, 50,  

  ILI932X_POW_CTRL4, 0x1800,
  ILI932X_POW_CTRL7, 0x002A,

  TFTLCD_DELAYCMD,50,
  
  ILI932X_GAMMA_CTRL1, 0x0000,    
  ILI932X_GAMMA_CTRL2, 0x0000, 
  ILI932X_GAMMA_CTRL3, 0x0000,
  ILI932X_GAMMA_CTRL4, 0x0206,   
  ILI932X_GAMMA_CTRL5, 0x0808,  
  ILI932X_GAMMA_CTRL6, 0x0007,  
  ILI932X_GAMMA_CTRL7, 0x0201,
  ILI932X_GAMMA_CTRL8, 0x0000,  
  ILI932X_GAMMA_CTRL9, 0x0000,  
  ILI932X_GAMMA_CTRL10, 0x0000,  
 
  ILI932X_GRAM_HOR_AD, 0x0000,  
  ILI932X_GRAM_VER_AD, 0x0000,  
  ILI932X_HOR_START_AD, 0x0000,
  ILI932X_HOR_END_AD, 0x00EF,
  ILI932X_VER_START_AD, 0X0000,
  ILI932X_VER_END_AD, 0x013F,
   

  ILI932X_GATE_SCAN_CTRL1, 0xA700,     // Driver Output Control (R60h)
  ILI932X_GATE_SCAN_CTRL2, 0x0003,     // Driver Output Control (R61h)
  ILI932X_GATE_SCAN_CTRL3, 0x0000,     // Driver Output Control (R62h)

  ILI932X_PANEL_IF_CTRL1, 0X0010,     // Panel Interface Control 1 (R90h)
  ILI932X_PANEL_IF_CTRL2, 0X0000,
  ILI932X_PANEL_IF_CTRL3, 0X0003,
  ILI932X_PANEL_IF_CTRL4, 0X1100,
  ILI932X_PANEL_IF_CTRL5, 0X0000,
  ILI932X_PANEL_IF_CTRL6, 0X0000,

  // Display On
  ILI932X_DISP_CTRL1, 0x0133,     // Display Control (R07h)
};
//Initalizes the LCD
void begin(uint16_t id) {
  uint16_t a, d;

  constructor(TFTWIDTH, TFTHEIGHT);
  //fprintf(stdout,"Constructor was called\n\r");

  reset();
  driver = id;

  if (id == 0x9325 || id == 0x9328) {
    for (uint8_t i = 0; i < sizeof(ILI932x_regValues) / 4; i++) {
      a = pgm_read_word(ILI932x_regValues + i*2);
      d = pgm_read_word(ILI932x_regValues + i*2 + 1);
	  //fprintf(stdout, "Value of i is %d\n\r", i);
	  //fprintf(stdout,"Value of a is: %x\n\r", a);
	  //fprintf(stdout,"Value of d is: %x\n\r", d);
      
      if (a == 0xFF) {
		_delay_ms(60); //_delay_ms(d);
      } else {
		writeRegister16(a, d);
		//_delay_ms(50);
	//Serial.print("addr: "); Serial.print(a); 
	//Serial.print(" data: "); Serial.println(d, HEX);
      }
    }
  } else if (id == 0x7575) {
    for (uint8_t i = 0; i < sizeof(HX8347G_regValues) / 4; i++) {
      a = pgm_read_word(HX8347G_regValues + i*2);
      d = pgm_read_word(HX8347G_regValues + i*2 + 1);
      
      if (a == 0xFF) {
		_delay_ms(200);//_delay_ms(d);
      } 
	  else {
		writeRegister8(a, d);
	//Serial.print("addr: "); Serial.print(a); 
	//Serial.print(" data: "); Serial.println(d, HEX);
      }
    }
  }
  //fprintf(stdout,"Begin function completed!\n\r");

}
/********************************* low level pin initialization */
//LCD constructor
void Adafruit_TFTLCD(uint8_t cs, uint8_t cd, uint8_t wr, uint8_t rd, uint8_t reset) {
  _cs = cs;
  _cd = cd;
  _wr = wr;
  _rd = rd;
  _reset = reset;
  
  rotation = 0;
  _width = TFTWIDTH;
  _height = TFTHEIGHT;

  // disable the LCD
  //digitalWrite(_cs, HIGH);
  //pinMode(_cs, OUTPUT); 
  PORTC =  (1<<PINC0);
  DDRC  =  (1<<PINC0); 
  //PORTC = (1<<PORTC0) |  (1<<PORTC1) | (1<<PORTC2) | (1<<PORTC3) | (1<<PORTC4);
  //DDRC = (1<<PORTC0) |  (1<<PORTC1) | (1<<PORTC2) | (1<<PORTC3) | (1<<PORTC4);
  
  
  //digitalWrite(_cd, HIGH);
  //pinMode(_cd, OUTPUT);
  PORTC |= (1<<PINC1);
  DDRC  |= (1<<PINC1);  
  
  //digitalWrite(_wr, HIGH);
  //pinMode(_wr, OUTPUT);
  PORTC |= (1<<PINC2);
  DDRC  |= (1<<PINC2);  
  
  //digitalWrite(_rd, HIGH);
  //pinMode(_rd, OUTPUT);
  PORTC |= (1<<PINC3);
  DDRC  |= (1<<PINC3);  

  //digitalWrite(_reset, HIGH); 
  //pinMode(_reset, OUTPUT); 
  PORTC |= (1<<PINC4);
  DDRC  |= (1<<PINC4);

  //csport = digitalPinToPort(_cs);
  //cdport = digitalPinToPort(_cd);
  //wrport = digitalPinToPort(_wr);
  //rdport = digitalPinToPort(_rd);

  //cspin = digitalPinToBitMask(_cs);
  //cdpin = digitalPinToBitMask(_cd);
  //wrpin = digitalPinToBitMask(_wr);
  //rdpin = digitalPinToBitMask(_rd);

  //wrportreg = portOutputRegister(wrport);

  cursor_y = cursor_x = 0;
  textsize = 1;
  textcolor = 0xFFFF;
}


/********************************** low level pin interface */
//Resets the LCD
void reset(void) {
  /*if (_reset)
    digitalWrite(_reset, LOW);
  delay(2); 
  if (_reset)
    digitalWrite(_reset, HIGH); */
	//fprintf(stdout, "I am resetting!\n\r");
	PORTC &=~(1<<PINC4);
	_delay_ms(2);
	PORTC |=(1<<PINC4);
	//fprintf(stdout, "I am done resetting!\n\r");

  // resync
  writeData(0);
  writeData(0);
  writeData(0);  
  writeData(0);
} 

//Sets the data pins to outputs
void setWriteDir(void) {
  DATADDR2 |= DATA2_MASK;
  DATADDR1 |= DATA1_MASK;
}
//Sets the data pins to inputs
void setReadDir(void) {
  DATADDR2 &= ~DATA2_MASK;
  DATADDR1 &= ~DATA1_MASK;
}
//Writes 8-bits of data
void write8(uint8_t d) {

  DATAPORT2 = (DATAPORT2 & DATA1_MASK) | 
    (d & DATA2_MASK);
  DATAPORT1 = (DATAPORT1 & DATA2_MASK) | 
    (d & DATA1_MASK); // top 6 bits
}
//Reads 8-bits of data
uint8_t read8(void) {
 uint8_t d;

 d = DATAPIN1 & DATA1_MASK; 
 d |= DATAPIN2 & DATA2_MASK; 


 return d;
}

/********************************** low level readwrite interface */

// the C/D pin is high during write
void writeData(uint16_t data) {

  //*wrportreg |=  wrpin;                   //digitalWrite(_wr, HIGH);
  //*portOutputRegister(csport) &= ~cspin;  //digitalWrite(_cs, LOW);
  //*portOutputRegister(cdport) |= cdpin;   //digitalWrite(_cd, HIGH);
  //*portOutputRegister(rdport) |= rdpin;   //digitalWrite(_rd, HIGH);
  PORTC &= ~(1<<PINC0);
  PORTC |= (1<<PINC1) | (1<<PINC2) | (1<<PINC3);

  setWriteDir();
  write8(data >> 8);
  
  //*wrportreg &= ~wrpin;    //digitalWrite(_wr, LOW);
  PORTC &= ~(1<<PINC2);
  //*wrportreg |=  wrpin;    //digitalWrite(_wr, HIGH);
  PORTC |= (1<<PINC2);

  write8(data);

  //*wrportreg &= ~wrpin;   //digitalWrite(_wr, LOW);
  PORTC &= ~(1<<PINC2);
  //*wrportreg |=  wrpin;   //digitalWrite(_wr, HIGH);
  PORTC |= (1<<PINC2);
  //*portOutputRegister(csport) |= cspin;    //digitalWrite(_cs, HIGH);
  PORTC |= (1<<PINC0);
}

//Writes 8-bit value to a register
void writeRegister8(uint8_t addr, uint8_t data) {
  //*wrportreg |=  wrpin;                    //digitalWrite(_wr, HIGH);
  //*portOutputRegister(csport) &= ~cspin;   //digitalWrite(_cs, LOW);
  //*portOutputRegister(cdport) &= ~cdpin;   //digitalWrite(_cd, LOW); ???? comment said high
  //*portOutputRegister(rdport) |= rdpin;    //digitalWrite(_rd, HIGH);
  PORTC |= (1<<PINC2) | (1<<PINC3);
  PORTC &= ~((1<<PINC0) | (1<<PINC1));

  setWriteDir();
  write8(addr);

  //*wrportreg &= ~wrpin;   //digitalWrite(_wr, LOW);
  PORTC &= ~(1<<PINC2);
  //*wrportreg |=  wrpin;   //digitalWrite(_wr, HIGH);
  PORTC |= (1<<PINC2);

  //*portOutputRegister(cdport) |= cdpin;     //digitalWrite(_cd, HIGH);
  PORTC |= (1<<PINC1);

  write8(data);

  //*wrportreg &= ~wrpin;                    //digitalWrite(_wr, LOW);
  PORTC &= ~(1<<PINC2);
  //*wrportreg |=  wrpin;                    //digitalWrite(_wr, HIGH);
  PORTC |= (1<<PINC2);
  //*portOutputRegister(csport) |= cspin;    //digitalWrite(_cs, HIGH);
  PORTC |= (1<<PINC0);
}

// this is a 'sped up' version, with no direction setting, or pin initialization
// not for external usage, but it does speed up stuff like a screen fill
void writeData_unsafe(uint16_t data) {
  write8(data >> 8);

  //*wrportreg &= ~wrpin;   //digitalWrite(_wr, LOW);
  PORTC &= ~(1<<PINC2);
  //*wrportreg |=  wrpin;   //digitalWrite(_wr, HIGH);
  PORTC |= (1<<PINC2);

  write8(data);

  //*wrportreg &= ~wrpin;   //digitalWrite(_wr, LOW);
  PORTC &= ~(1<<PINC2);
  //*wrportreg |=  wrpin;   //digitalWrite(_wr, HIGH);
  PORTC |= (1<<PINC2);
}

// the C/D pin is low during write
void writeCommand(uint16_t cmd) {
  //*wrportreg |=  wrpin;                    //digitalWrite(_wr, HIGH);
  //*portOutputRegister(csport) &= ~cspin;   //digitalWrite(_cs, LOW);
  //*portOutputRegister(cdport) &= ~cdpin;   //digitalWrite(_cd, LOW);
  //*portOutputRegister(rdport) |= rdpin;    //digitalWrite(_rd, HIGH);
  PORTC &= ~((1<<PINC0) | (1<<PINC1));
  PORTC |= (1<<PINC2) | (1<<PINC3);

  setWriteDir();
  write8(cmd >> 8);

  //*wrportreg &= ~wrpin;    //digitalWrite(_wr, LOW);
  PORTC &= ~(1<<PINC2);
  //*wrportreg |=  wrpin;    //digitalWrite(_wr, HIGH);
  PORTC |= (1<<PINC2);

  write8(cmd);


  //*wrportreg &= ~wrpin;    //digitalWrite(_wr, LOW);
  PORTC &= ~(1<<PINC2);
  //*wrportreg |=  wrpin;    //digitalWrite(_wr, HIGH);
  PORTC |= (1<<PINC2);
  //*portOutputRegister(csport) |= cspin;   //digitalWrite(_cs, HIGH);
  PORTC |= (1<<PINC0);
}
//Writes an 8-bit command
void writeCommand8(uint8_t cmd) {
  //*wrportreg |=  wrpin;                    //digitalWrite(_wr, HIGH);
  //*portOutputRegister(csport) &= ~cspin;   //digitalWrite(_cs, LOW);
  //*portOutputRegister(cdport) &= ~cdpin;   //digitalWrite(_cd, LOW);
  //*portOutputRegister(rdport) |= rdpin;    //digitalWrite(_rd, HIGH);
  PORTC &= ~((1<<PINC0) | (1<<PINC1)); 
  PORTC |= (1<<PINC2) | (1<<PINC3);


  setWriteDir();
  write8(cmd);

  //*wrportreg &= ~wrpin;   //digitalWrite(_wr, LOW);
  PORTC &= ~(1<<PINC2);
  //*wrportreg |=  wrpin;   //digitalWrite(_wr, HIGH);
  PORTC |= (1<<PINC2);
  //*portOutputRegister(csport) |= cspin;  //digitalWrite(_cs, HIGH);
  PORTC |= (1<<PINC0);
}
//Reads 16-bits of data and returns
uint16_t readData() {
 uint16_t d = 0;
 
  //*portOutputRegister(csport) &= ~cspin;  //digitalWrite(_cs, LOW);
  //*portOutputRegister(cdport) |= cdpin;   //digitalWrite(_cd, HIGH);
  //*portOutputRegister(rdport) |= rdpin;   //digitalWrite(_rd, HIGH);
  //*portOutputRegister(wrport) |= wrpin;   //digitalWrite(_wr, HIGH);
  PORTC &= ~(1<<PINC0);
  PORTC |= (1<<PINC1) |(1<<PINC2) | (1<<PINC3);
  
  setReadDir();

  //*portOutputRegister(rdport) &= ~rdpin;  //digitalWrite(_rd, LOW);
  PORTC &= ~(1<<PINC3);

  _delay_us(10);
  d = read8();
  d <<= 8;

  //*portOutputRegister(rdport) |= rdpin;   //digitalWrite(_rd, HIGH);
  PORTC |= (1<<PINC3);
  //*portOutputRegister(rdport) &= ~rdpin;  //digitalWrite(_rd, LOW);
  PORTC &= ~(1<<PINC3);

  _delay_us(10);
  d |= read8();

  //*portOutputRegister(rdport) |= rdpin;   //digitalWrite(_rd, HIGH);
  PORTC |= (1<<PINC3);
  //*portOutputRegister(csport) |= cspin;   //digitalWrite(_cs, HIGH);
  PORTC |= (1<<PINC0);
   
  return d;
}


/************************************* medium level data reading/writing */
//Reads a 16-bit register
uint16_t readRegister(uint16_t addr) {
   writeCommand(addr);
   return readData();
}

//Writes 16-bits to a register
void writeRegister16(uint16_t addr, uint16_t data) {
  writeCommand(addr);
  writeData(data);
}
//Drawsa string to the current cursor position. Input must end with '\n'
void drawString(char string[])
{
	int i =0;
	while(string[i] != '\n')
	{
			drawChar(cursor_x,cursor_y, string[i],textcolor, textbgcolor, textsize);
			cursor_x+=(textsize*6);
			i++;
	}
}
//Taken from the Arduino library
//Maps x from one range to another
long map(long x, long in_min, long in_max, long out_min, long out_max)
{

  return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
}
