/***********************************
This is a our graphics core library, for all our displays. 
We'll be adapting all the
existing libaries to use this core to make updating, support 
and upgrading easier!

Adafruit invests time and resources providing this open source code, 
please support Adafruit and open-source hardware by purchasing 
products from Adafruit!

Written by Limor Fried/Ladyada  for Adafruit Industries.  
BSD license, check license.txt for more information
All text above must be included in any redistribution
****************************************/

#ifndef _ADAFRUIT_GFX_H
#define _ADAFRUIT_GFX_H

/*#if ARDUINO >= 100
 #include "Arduino.h"
 #include "Print.h"
#else
 #include "WProgram.h"
#endif */

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>

//Definitions from TFT
#define ILI932X_START_OSC			0x00
#define ILI932X_DRIV_OUT_CTRL		0x01
#define ILI932X_DRIV_WAV_CTRL		0x02
#define ILI932X_ENTRY_MOD			0x03
#define ILI932X_RESIZE_CTRL			0x04
#define ILI932X_DISP_CTRL1			0x07
#define ILI932X_DISP_CTRL2			0x08
#define ILI932X_DISP_CTRL3			0x09
#define ILI932X_DISP_CTRL4			0x0A
#define ILI932X_RGB_DISP_IF_CTRL1	0x0C
#define ILI932X_FRM_MARKER_POS		0x0D
#define ILI932X_RGB_DISP_IF_CTRL2	0x0F
#define ILI932X_POW_CTRL1			0x10
#define ILI932X_POW_CTRL2			0x11
#define ILI932X_POW_CTRL3			0x12
#define ILI932X_POW_CTRL4			0x13
#define ILI932X_GRAM_HOR_AD			0x20
#define ILI932X_GRAM_VER_AD			0x21
#define ILI932X_RW_GRAM				0x22
#define ILI932X_POW_CTRL7			0x29
#define ILI932X_FRM_RATE_COL_CTRL	0x2B
#define ILI932X_GAMMA_CTRL1			0x30
#define ILI932X_GAMMA_CTRL2			0x31
#define ILI932X_GAMMA_CTRL3			0x32
#define ILI932X_GAMMA_CTRL4			0x35 
#define ILI932X_GAMMA_CTRL5			0x36
#define ILI932X_GAMMA_CTRL6			0x37
#define ILI932X_GAMMA_CTRL7			0x38
#define ILI932X_GAMMA_CTRL8			0x39
#define ILI932X_GAMMA_CTRL9			0x3C
#define ILI932X_GAMMA_CTRL10			0x3D
#define ILI932X_HOR_START_AD			0x50
#define ILI932X_HOR_END_AD			0x51
#define ILI932X_VER_START_AD			0x52
#define ILI932X_VER_END_AD			0x53
#define ILI932X_GATE_SCAN_CTRL1		0x60
#define ILI932X_GATE_SCAN_CTRL2		0x61
#define ILI932X_GATE_SCAN_CTRL3		0x6A
#define ILI932X_PART_IMG1_DISP_POS	0x80
#define ILI932X_PART_IMG1_START_AD	0x81
#define ILI932X_PART_IMG1_END_AD		0x82
#define ILI932X_PART_IMG2_DISP_POS	0x83
#define ILI932X_PART_IMG2_START_AD	0x84
#define ILI932X_PART_IMG2_END_AD		0x85
#define ILI932X_PANEL_IF_CTRL1		0x90
#define ILI932X_PANEL_IF_CTRL2		0x92
#define ILI932X_PANEL_IF_CTRL3		0x93
#define ILI932X_PANEL_IF_CTRL4		0x95
#define ILI932X_PANEL_IF_CTRL5		0x97
#define ILI932X_PANEL_IF_CTRL6		0x98

#define HX8347G_COLADDRSTART2 0x02
#define HX8347G_COLADDRSTART1 0x03
#define HX8347G_COLADDREND2 0x04
#define HX8347G_COLADDREND1 0x05

#define HX8347G_ROWADDRSTART2 0x06
#define HX8347G_ROWADDRSTART1 0x07
#define HX8347G_ROWADDREND2 0x08
#define HX8347G_ROWADDREND1 0x09


#define TFTLCD_DELAYCMD                 0xFF

#define swap(a, b) { int16_t t = a; a = b; b = t; }

//Functions from GFX

  void constructor(uint16_t w, uint16_t h);

  // this must be defined by the subclass
  //Changes the color of a pixel
  void drawPixel(uint16_t x, uint16_t y, uint16_t color);

  // these are 'generic' drawing functions, so we can share them!
  //Draws a line from x0,y0 to x1,y1 in the specified color
  void drawLine(int16_t x0, int16_t y0, int16_t x1, int16_t y1, 
		uint16_t color);
  //Draws a verticle line of length h starting at x,y
  void drawFastVLine(uint16_t x, uint16_t y, uint16_t h, uint16_t color);
  //Draws a horizontal line of length h starting at x,y
  void drawFastHLine(uint16_t x, uint16_t y, uint16_t w, uint16_t color);
  //Draws the perimeter of a rectangle at x,y of width w and height h.
  void drawRect(uint16_t x, uint16_t y, uint16_t w, uint16_t h, 
		uint16_t color);
  //Draws a rectangle at x,y of width w and height h
  void fillRect(uint16_t x, uint16_t y, uint16_t w, uint16_t h, 
		uint16_t color);
  //Fills the screen with the specified color
  void fillScreen(uint16_t color);

  //Draws the perimeter of a circle with center x0,y0 and radius r
  void drawCircle(uint16_t x0, uint16_t y0, uint16_t r, 
		  uint16_t color);
  //Helper function.  Used to draw the corners of a rounded rectangle
  void drawCircleHelper(uint16_t x0, uint16_t y0,
			uint16_t r, uint8_t cornername, uint16_t color);
  //Draws a circle with center x,y and radius r.
  void fillCircle(uint16_t x0, uint16_t y0, uint16_t r, 
		  uint16_t color);
  //Draws the left half of a circle with center x,y and radius r
  void fillLeftHalfCircle(uint16_t x0, uint16_t y0, uint16_t r, 
			      uint16_t color);
  //Draws the right half of a circle with center x,y and radius r
  void fillRightHalfCircle(uint16_t x0, uint16_t y0, uint16_t r, 
			      uint16_t color);
  //Draws the top half of a circle with center x,y and radius r
  void fillTopHalfCircle(uint16_t x0, uint16_t y0, uint16_t r, 
			      uint16_t color);
  //Draws the bottom half of a circle with center x,y and radius r
  void fillBottomHalfCircle(uint16_t x0, uint16_t y0, uint16_t r, 
			      uint16_t color);
  //Helper function for fillCircle and fillRoundedRect
  void fillCircleHelper(uint16_t x0, uint16_t y0, uint16_t r,
		      uint8_t cornername, uint16_t delta, uint16_t color);
  //Draws the perimeter of a triangle with corners x0,y0, x1,y1 and x2,y2
  void drawTriangle(uint16_t x0, uint16_t y0, uint16_t x1, uint16_t y1,
		    uint16_t x2, uint16_t y2, uint16_t color);
  //Draws a triangle with corners x0,y0, x1,y1 and x2,y2
  void fillTriangle(uint16_t x0, uint16_t y0, uint16_t x1, uint16_t y1,
		    uint16_t x2, uint16_t y2, uint16_t color);
  //Draws the perimeter of a rectangle with rounded corners with center x0,y0 with
  //width w and height h.  The corners have a radius r.
  void drawRoundRect(uint16_t x0, uint16_t y0, uint16_t w, uint16_t h,
		     uint16_t radius, uint16_t color);
  //Draws the a rectangle with rounded corners with center x0,y0 with
  //width w and height h.  The corners have a radius r.
  void fillRoundRect(uint16_t x0, uint16_t y0, uint16_t w, uint16_t h,
		     uint16_t radius, uint16_t color);
 //Draws a bitmap from a location in program memory.
 //Can be used to read an image from an SD card
  void drawBitmap(uint16_t x, uint16_t y, 
		  const uint8_t *bitmap, uint16_t w, uint16_t h,
		  uint16_t color);
  //Draws a character c to the positon x,y.
  //Text size is an integer >=1
  void drawChar(uint16_t x, uint16_t y, char c,
		uint16_t color, uint16_t bg, uint8_t size);

  //Writes a character to the screen
  void write(uint8_t);
  //Sets the text cursor to x,y
  void setCursor(uint16_t x, uint16_t y);
  //Sets the text color
  void setTextColor(uint16_t c);
  //Sets the text color and background color
  void setTextColor2(uint16_t c, uint16_t bg);
  //Sets the text size
  void setTextSize(uint8_t s);

  //Maps the value of x from one range to a different range (Thanks Arduino)
  long map(long x, long in_min, long in_max, long out_min, long out_max);
  //Draws a string to the screen at the current cursor position
  //The string must end with '\n'
  void drawString(char string[]);

  //Returns the height of the LCD in pixels
  uint16_t height(void);
  //Returns the width of the screen in pixels
  uint16_t width(void);

  //Sets the rotation of the screen (0,1,2,3)
  void setRotation(uint8_t r);
  //Returns the rotation of the screen
  uint8_t getRotation(void);

  uint16_t WIDTH, HEIGHT;       // this is the 'raw' display w/h - never changes
  uint16_t _width, _height;     // dependant on rotation
  uint16_t cursor_x, cursor_y, textcolor, textbgcolor;
  uint8_t textsize;
  uint8_t rotation;



//Functions from TFT
  void Adafruit_TFTLCD(uint8_t cs, uint8_t cd, uint8_t wr, uint8_t rd, uint8_t reset);

  //Converts 24-bit RGB value to a 16-bit RGB value
  uint16_t Color565(uint8_t r, uint8_t g, uint8_t b);

  // drawing primitives!
  //Changes the color of the pixel x,y
  void drawPixel(uint16_t x, uint16_t y, uint16_t color);
  //Fills the screen with specified color
  void fillScreen(uint16_t color);
  //Draws a verticle line
  void drawFastVLine(uint16_t x0, uint16_t y0, uint16_t length, uint16_t color);
  //Draws a horizontal line
  void drawFastHLine(uint16_t x0, uint16_t y0, uint16_t length, uint16_t color);

  //Draws a rectangle at position x0,y0 of width w and height h
  void fillRect(uint16_t x0, uint16_t y0, uint16_t w, uint16_t h, uint16_t color);

  // commands
  //Initialized the LCD controller. ID is ID of the controller
  void begin(uint16_t id ); //= 0x9325
  //Sets cursor to x,y
  void goTo(int x, int y);
  //Resets the LCD controller
  void reset(void);

  /* low level */

  //Writes 16-bit data to controller
  void writeData(uint16_t d);
  //Writes a 16-bit command
  void writeCommand(uint16_t c);
  //Writes an 8-bit command
  void writeCommand8(uint8_t c);
  //Reads 16-bits of data
  uint16_t readData(void);
  //Reads 16-bits of data from the specified register
  uint16_t readRegister(uint16_t addr);
  //Writes 8-bits of data to address a
  void writeRegister8(uint8_t a, uint8_t d);
  //Writes 16-bits of data to address addr
  void writeRegister16(uint16_t addr, uint16_t data);

  static const uint16_t TFTWIDTH = 240;
  static const uint16_t TFTHEIGHT = 320;

  void writeData_unsafe(uint16_t d);

  //Sets the data pins to outputs
  void setWriteDir(void);
  //Sets the data pins to inputs
  void setReadDir(void);
  //Writes 8-bit data d
  void write8(uint8_t d);

  uint16_t driver;

  void drawFastLine(uint16_t x0, uint16_t y0, uint16_t l, uint16_t color, uint8_t flag);
  uint8_t read8(void);

  uint8_t _cs, _cd, _reset, _wr, _rd;

  uint8_t csport, cdport, wrport, rdport;
  uint8_t cspin, cdpin, wrpin, rdpin;

#endif
