#include "Adafruit_GFX.h"
#include "glcdfont.c"
#include <avr/pgmspace.h>
#include <stdlib.h>
#include <util/delay.h>
#include "uart.h"
#include "TouchScreen.h"
#include "Programs.h"

 //Screen parameters
 #define BOXSIZE 40
 #define MINPRESSURE 200
 #define MAXPRESSURE 1000
 #define PENRADIUS 4
 #define TS_MINX 150
 #define TS_MINY 120
 #define TS_MAXX 920
 #define TS_MAXY 940

  //General colors
 #define	BLACK           0x0000
 #define 	WHITE           0xFFFF
 #define	RED             0xF800
 #define 	YELLOW          0xFFE0 
 #define	GREEN           0x07E0
 #define 	CYAN            0x07FF
 #define	BLUE            0x001F
 #define 	MAGENTA         0xF81F
 #define	GREY			50712
 #define    ORANGE			64800
 #define	PINK			63733
 #define	BROWN			35362

 //Colors for Free Draw
 //State 0 Colors
 #define	S0C1            0xF800
 #define 	S0C2          	0xFFE0 
 #define	S0C3           	0x07E0
 #define 	S0C4            0x07FF
 #define	S0C5           	0x001F
 #define 	S0C6         	0xF81F

 //State 1 Colors
 #define	S1C1            0x1234
 #define 	S1C2          	0x5678 
 #define	S1C3           	0x9ABC
 #define 	S1C4            0xDEF0
 #define	S1C5           	0xF0F0
 #define 	S1C6         	0x0F0F

 //State 2 Colors
 #define	S2C1            0x4321
 #define 	S2C2          	0xFFFF 
 #define	S2C3           	0xCBA9
 #define 	S2C4            0x0FEC
 #define	S2C5           	0x5555
 #define 	S2C6         	0x9999

  //State 3 Colors
 #define	S3C1            0x1357
 #define 	S3C2          	0x2468 
 #define	S3C3           	0x3579
 #define 	S3C4            0x468A
 #define	S3C5           	0x579B
 #define 	S3C6         	0x68AC

void free_draw()
{
    //Touch Variables
    int x;
	int y;
	int yy;
	uint16_t z;
	int pen_size = PENRADIUS;
	//Free Draw variables
	int currentcolor;
	int oldcolor;
	//Start on first set of colors
	int color_state=0;
	//Start on first color
	currentcolor=S0C1;
	//Old color is different from current color
	oldcolor=S0C2;
	//Set default pen radius
	pen_size=PENRADIUS;
	fillScreen(BLACK);
	//Draw colors at top of screen
  	fillRect(0, 0, BOXSIZE, BOXSIZE, S0C1);
  	fillRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, S0C2);
  	fillRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, S0C3);
  	fillRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, S0C4);
  	fillRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, S0C5);
  	fillRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, S0C6);
	//Draw boxes for tool bar
  	fillRect(0, BOXSIZE, BOXSIZE, BOXSIZE, WHITE);
  	fillRect(BOXSIZE, BOXSIZE, BOXSIZE, BOXSIZE, WHITE);
  	fillRect(BOXSIZE*2, BOXSIZE, BOXSIZE, BOXSIZE, WHITE);
  	fillRect(BOXSIZE*3, BOXSIZE, BOXSIZE, BOXSIZE, WHITE);
  	fillRect(BOXSIZE*4, BOXSIZE, BOXSIZE, BOXSIZE, WHITE);
  	fillRect(BOXSIZE*5, BOXSIZE, BOXSIZE, BOXSIZE, WHITE);
	//Put white border around selected color
  	drawRect(0, 0, BOXSIZE, BOXSIZE, WHITE);
	//Put black border around toolbar boxes
  	drawRect(0, BOXSIZE, BOXSIZE, BOXSIZE, BLACK);
  	drawRect(BOXSIZE, BOXSIZE, BOXSIZE, BOXSIZE, BLACK);
  	drawRect(BOXSIZE*2, BOXSIZE, BOXSIZE, BOXSIZE, BLACK);
  	drawRect(BOXSIZE*3, BOXSIZE, BOXSIZE, BOXSIZE, BLACK);
  	drawRect(BOXSIZE*4, BOXSIZE, BOXSIZE, BOXSIZE, BLACK);
  	drawRect(BOXSIZE*5, BOXSIZE, BOXSIZE, BOXSIZE, BLACK);
  	fillCircle(BOXSIZE/2,BOXSIZE+ BOXSIZE/2,5,BLACK);
  	fillCircle(BOXSIZE + BOXSIZE/2,BOXSIZE+ BOXSIZE/2,10,BLACK);
	//Set text color and size
  	setTextColor(RED);
  	setCursor(BOXSIZE*2+10,BOXSIZE+7);
  	setTextSize(4);
	//Draw characters on toolboxes
  	drawString("X\n");
  	setCursor(BOXSIZE*3+10,BOXSIZE+7);
  	drawString("E\n");
  	setCursor(BOXSIZE*4+7,BOXSIZE+7);
  	drawString("<\n");
  	setCursor(BOXSIZE*5+10,BOXSIZE+7);
  	drawString(">\n");
	while(1)
	{
		//Read X,Y raw positons and pressure
		x=readTouchX();
		y=readTouchY();
		yy=y;
		z=pressure();
		//Convert raw X,Y to screen positions
		x=map(x,TS_MINX, TS_MAXX, TFTWIDTH, 0);
		y=map(y,TS_MINY, TS_MAXY, TFTHEIGHT, 0);
		//If user touches top of screen, return to main menu
		if (yy>TS_MAXX+10)
		{
			return;
		}
		//If valid touch occured
		else if (z>250 && z<750)
		{
			//If top color boxes were touched, color is going to change
			if (y < BOXSIZE) {
				//Save old color
       			oldcolor = currentcolor;
 				//If in first color state
    			if (color_state==0)
				{
					//Set color to color 1 in state 0
       				if (x < BOXSIZE) { 
         				currentcolor = S0C1; 
         				drawRect(0, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Set color to color 2 in state 0 
       				else if (x < BOXSIZE*2) {
         				currentcolor = S0C2; 
         				drawRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Set color to color 3 in state 0
       				else if (x < BOXSIZE*3) {
         				currentcolor = S0C3; 
         				drawRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Set color to color 4 in state 0
       				else if (x < BOXSIZE*4) {
         				currentcolor = S0C4; 
         				drawRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Set color to color 5 in state 0
       				else if (x < BOXSIZE*5) {
         				currentcolor = S0C5; 
         				drawRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Set color to color 6 in state 0
       				else if (x < BOXSIZE*6) {
         				currentcolor = S0C6; 
        				drawRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Remove white border from previously selected color
					if (oldcolor != currentcolor) {
          				if (oldcolor == S0C1) fillRect(0, 0, BOXSIZE, BOXSIZE, S0C1);
          				if (oldcolor == S0C2) fillRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, S0C2);
          				if (oldcolor == S0C3) fillRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, S0C3);
          				if (oldcolor == S0C4) fillRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, S0C4);
          				if (oldcolor == S0C5) fillRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, S0C5);
          				if (oldcolor == S0C6) fillRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, S0C6);
       				}
	   			}
				//If in second color state
	   			else if (color_state==1)
	   			{
					//Set color to color 1 in state 1
					if (x < BOXSIZE) { 
         				currentcolor = S1C1; 
         				drawRect(0, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Set color to color 2 in state 1 
       				else if (x < BOXSIZE*2) {
         				currentcolor = S1C2; 
         				drawRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Set color to color 3 in state 1 
       				else if (x < BOXSIZE*3) {
         				currentcolor = S1C3; 
         				drawRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Set color to color 4 in state 1 
       				else if (x < BOXSIZE*4) {
         				currentcolor = S1C4; 
         				drawRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Set color to color 5 in state 1 
       				else if (x < BOXSIZE*5) {
         				currentcolor = S1C5; 
         				drawRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Set color to color 6 in state 1 
       				else if (x < BOXSIZE*6) {
         				currentcolor = S1C6; 
        				drawRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Remove white border from previously selected color
					if (oldcolor != currentcolor) {
          				if (oldcolor == S1C1) fillRect(0, 0, BOXSIZE, BOXSIZE, S1C1);
          				if (oldcolor == S1C2) fillRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, S1C2);
          				if (oldcolor == S1C3) fillRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, S1C3);
          				if (oldcolor == S1C4) fillRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, S1C4);
          				if (oldcolor == S1C5) fillRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, S1C5);
          				if (oldcolor == S1C6) fillRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, S1C6);
       				}
	   			}
				//If in third color state
	   			else if (color_state==2)
	   			{
					//Set color to color 1 in state 2
					if (x < BOXSIZE) { 
         				currentcolor = S2C1; 
         				drawRect(0, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Set color to color 2 in state 2 
       				else if (x < BOXSIZE*2) {
         				currentcolor = S2C2; 
         				drawRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Set color to color 3 in state 2
       				else if (x < BOXSIZE*3) {
         				currentcolor = S2C3; 
         				drawRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Set color to color 4 in state 2
       				else if (x < BOXSIZE*4) {
         				currentcolor = S2C4; 
         				drawRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Set color to color 5 in state 2
       				else if (x < BOXSIZE*5) {
         				currentcolor = S2C5; 
         				drawRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Set color to color 6 in state 2
       				else if (x < BOXSIZE*6) {
         				currentcolor = S2C6; 
        				drawRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Erase white border around previous color
					if (oldcolor != currentcolor) {
          				if (oldcolor == S2C1) fillRect(0, 0, BOXSIZE, BOXSIZE, S2C1);
          				if (oldcolor == S2C2) fillRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, S2C2);
          				if (oldcolor == S2C3) fillRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, S2C3);
          				if (oldcolor == S2C4) fillRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, S2C4);
          				if (oldcolor == S2C5) fillRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, S2C5);
          				if (oldcolor == S2C6) fillRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, S2C6);
       				}
	   			}
				//If in fourth color state
	   			else if (color_state==3)
	   			{
					//Set color to color 1 in state 3
					if (x < BOXSIZE) { 
         				currentcolor = S3C1; 
         				drawRect(0, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Set color to color 2 in state 3 
       				else if (x < BOXSIZE*2) {
         				currentcolor = S3C2; 
         				drawRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Set color to color 3 in state 3
       				else if (x < BOXSIZE*3) {
         				currentcolor = S3C3; 
         				drawRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Set color to color 4 in state 3
       				else if (x < BOXSIZE*4) {
         				currentcolor = S3C4; 
         				drawRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Set color to color 5 in state 3
       				else if (x < BOXSIZE*5) {
         				currentcolor = S3C5; 
         				drawRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Set color to color 6 in state 3
       				else if (x < BOXSIZE*6) {
         				currentcolor = S3C6; 
        				drawRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, WHITE);
       				}
					//Erase white border around previous color
					if (oldcolor != currentcolor) {
          				if (oldcolor == S3C1) fillRect(0, 0, BOXSIZE, BOXSIZE, S3C1);
          				if (oldcolor == S3C2) fillRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, S3C2);
          				if (oldcolor == S3C3) fillRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, S3C3);
          				if (oldcolor == S3C4) fillRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, S3C4);
          				if (oldcolor == S3C5) fillRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, S3C5);
          				if (oldcolor == S3C6) fillRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, S3C6);
       				}
	   			}
    		}
			//If user selects button in toolbar
			if (y>BOXSIZE && y<(BOXSIZE+BOXSIZE))
			{
	   			//Decrease pen size
	   			if (x < BOXSIZE) {
	     			if(pen_size > 1)
		 			{ 
         				pen_size--;
						drawRect(0, BOXSIZE, BOXSIZE, BOXSIZE, BLUE);
						drawRect(1, BOXSIZE+1, BOXSIZE-2, BOXSIZE-2, BLUE);
						drawRect(2, BOXSIZE+2, BOXSIZE-3, BOXSIZE-3, BLUE);
						_delay_ms(200);
						drawRect(0, BOXSIZE, BOXSIZE, BOXSIZE, BLACK);
						drawRect(1, BOXSIZE+1, BOXSIZE-2, BOXSIZE-2, WHITE);
						drawRect(2, BOXSIZE+2, BOXSIZE-3, BOXSIZE-3, WHITE);
		 			}
       			} 
	   			//Increase pen size
       			else if (x < BOXSIZE*2) {
	     			if(pen_size < 15)
		 			{
         				pen_size++;
						drawRect(BOXSIZE, BOXSIZE, BOXSIZE, BOXSIZE, BLUE);
						drawRect(BOXSIZE+1, BOXSIZE+1, BOXSIZE-2, BOXSIZE-2, BLUE);
						drawRect(BOXSIZE+2, BOXSIZE+2, BOXSIZE-3, BOXSIZE-3, BLUE);
						_delay_ms(200);
						drawRect(BOXSIZE, BOXSIZE, BOXSIZE, BOXSIZE, BLACK);
						drawRect(BOXSIZE+1, BOXSIZE+1, BOXSIZE-2, BOXSIZE-2, WHITE);
						drawRect(BOXSIZE+2, BOXSIZE+2, BOXSIZE-3, BOXSIZE-3, WHITE);
		 			}
	   			}
       			//Erase drawing
	   			else if (x < BOXSIZE*3) {
	   				drawRect(BOXSIZE+BOXSIZE, BOXSIZE, BOXSIZE, BOXSIZE, BLUE);
					drawRect(BOXSIZE+BOXSIZE+1, BOXSIZE+1, BOXSIZE-2, BOXSIZE-2, BLUE);
					drawRect(BOXSIZE+BOXSIZE+2, BOXSIZE+2, BOXSIZE-3, BOXSIZE-3, BLUE);
					fillRect(0, BOXSIZE+BOXSIZE, TFTWIDTH, TFTHEIGHT-BOXSIZE-BOXSIZE, BLACK);
					drawRect(BOXSIZE+BOXSIZE, BOXSIZE, BOXSIZE, BOXSIZE, BLACK);
					drawRect(BOXSIZE+BOXSIZE+1, BOXSIZE+1, BOXSIZE-2, BOXSIZE-2, WHITE);
					drawRect(BOXSIZE+BOXSIZE+2, BOXSIZE+2, BOXSIZE-3, BOXSIZE-3, WHITE);
	   			}
	   			//Eraser (changes pen color to black)
	   			else if (x < BOXSIZE*4) {
	   				drawRect(BOXSIZE*3, BOXSIZE, BOXSIZE, BOXSIZE, BLUE);
					drawRect(BOXSIZE*3+1, BOXSIZE+1, BOXSIZE-2, BOXSIZE-2, BLUE);
					drawRect(BOXSIZE*3+2, BOXSIZE+2, BOXSIZE-3, BOXSIZE-3, BLUE);
					_delay_ms(200);
					drawRect(BOXSIZE*3, BOXSIZE, BOXSIZE, BOXSIZE, BLACK);
					drawRect(BOXSIZE*3+1, BOXSIZE+1, BOXSIZE-2, BOXSIZE-2, WHITE);
					drawRect(BOXSIZE*3+2, BOXSIZE+2, BOXSIZE-3, BOXSIZE-3, WHITE);
					oldcolor=currentcolor;
					currentcolor=BLACK;
					//Erase white border from previously selected color
					if (oldcolor == S0C1 || oldcolor == S1C1 || oldcolor == S2C1 || oldcolor == S3C1) drawRect(0, 0, BOXSIZE, BOXSIZE, RED);
          			if (oldcolor == S0C2 || oldcolor == S1C2 || oldcolor == S2C2 || oldcolor == S3C2) drawRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, YELLOW);
          			if (oldcolor == S0C3 || oldcolor == S1C3 || oldcolor == S2C3 || oldcolor == S3C3) drawRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, GREEN);
          			if (oldcolor == S0C4 || oldcolor == S1C4 || oldcolor == S2C4 || oldcolor == S3C4) drawRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, CYAN);
          			if (oldcolor == S0C5 || oldcolor == S1C5 || oldcolor == S2C5 || oldcolor == S3C5) drawRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, BLUE);
          			if (oldcolor == S0C6 || oldcolor == S1C6 || oldcolor == S2C6 || oldcolor == S3C6) drawRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, MAGENTA);
	   			}
				//Change color state to the next lowest state
	   			else if (x< BOXSIZE*5)
	   			{
	   				drawRect(BOXSIZE*4, BOXSIZE, BOXSIZE, BOXSIZE, BLUE);
					drawRect(BOXSIZE*4+1, BOXSIZE+1, BOXSIZE-2, BOXSIZE-2, BLUE);
					drawRect(BOXSIZE*4+2, BOXSIZE+2, BOXSIZE-3, BOXSIZE-3, BLUE);
					_delay_ms(100);
					drawRect(BOXSIZE*4, BOXSIZE, BOXSIZE, BOXSIZE, BLACK);
					drawRect(BOXSIZE*4+1, BOXSIZE+1, BOXSIZE-2, BOXSIZE-2, WHITE);
					drawRect(BOXSIZE*4+2, BOXSIZE+2, BOXSIZE-3, BOXSIZE-3, WHITE);
					//Draw in colors for state 3
					if(color_state==0)
					{
						color_state=3;
						fillRect(0, 0, BOXSIZE, BOXSIZE, S3C1);
  						fillRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, S3C2);
  						fillRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, S3C3);
  						fillRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, S3C4);
  						fillRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, S3C5);
  						fillRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, S3C6);
						if (currentcolor == S3C1) drawRect(0, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S3C2) drawRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S3C3) drawRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S3C4) drawRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S3C5) drawRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S3C6) drawRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, WHITE);
					}
					//Draw in colors for state 0
					else if(color_state==1)
					{
						color_state=0;
						fillRect(0, 0, BOXSIZE, BOXSIZE, S0C1);
  						fillRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, S0C2);
  						fillRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, S0C3);
  						fillRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, S0C4);
  						fillRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, S0C5);
  						fillRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, S0C6);
						if (currentcolor == S0C1) drawRect(0, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S0C2) drawRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S0C3) drawRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S0C4) drawRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S0C5) drawRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S0C6) drawRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, WHITE);
					}
					//Draw in colors for state 1
					else if(color_state==2)
					{
						color_state=1;
						fillRect(0, 0, BOXSIZE, BOXSIZE, S1C1);
  						fillRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, S1C2);
  						fillRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, S1C3);
  						fillRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, S1C4);
  						fillRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, S1C5);
  						fillRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, S1C6);
						if (currentcolor == S1C1) drawRect(0, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S1C2) drawRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S1C3) drawRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S1C4) drawRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S1C5) drawRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S1C6) drawRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, WHITE);
					}
					//Draw in colors for state 2
					else if(color_state==3)
					{
						color_state=2;
						fillRect(0, 0, BOXSIZE, BOXSIZE, S2C1);
  						fillRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, S2C2);
  						fillRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, S2C3);
  						fillRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, S2C4);
  						fillRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, S2C5);
  						fillRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, S2C6);
						if (currentcolor == S2C1) drawRect(0, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S2C2) drawRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S2C3) drawRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S2C4) drawRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S2C5) drawRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S2C6) drawRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, WHITE);
					}
	   			}
				//Shift color state to next higher state
	   			else if (x< BOXSIZE*6)
	   			{
	   				drawRect(BOXSIZE*5, BOXSIZE, BOXSIZE, BOXSIZE, BLUE);
					drawRect(BOXSIZE*5+1, BOXSIZE+1, BOXSIZE-2, BOXSIZE-2, BLUE);
					drawRect(BOXSIZE*5+2, BOXSIZE+2, BOXSIZE-3, BOXSIZE-3, BLUE);
					_delay_ms(100);
					drawRect(BOXSIZE*5, BOXSIZE, BOXSIZE, BOXSIZE, BLACK);
					drawRect(BOXSIZE*5+1, BOXSIZE+1, BOXSIZE-2, BOXSIZE-2, WHITE);
					drawRect(BOXSIZE*5+2, BOXSIZE+2, BOXSIZE-3, BOXSIZE-3, WHITE);
					//Draw in colors for state 1
					if(color_state==0)
					{
						color_state=1;
						fillRect(0, 0, BOXSIZE, BOXSIZE, S1C1);
  						fillRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, S1C2);
  						fillRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, S1C3);
  						fillRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, S1C4);
  						fillRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, S1C5);
  						fillRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, S1C6);
						if (currentcolor == S1C1) drawRect(0, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S1C2) drawRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S1C3) drawRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S1C4) drawRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S1C5) drawRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S1C6) drawRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, WHITE);
					}
					//Draw in colors for state 2
					else if(color_state==1)
					{
						color_state=2;
						fillRect(0, 0, BOXSIZE, BOXSIZE, S2C1);
  						fillRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, S2C2);
  						fillRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, S2C3);
  						fillRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, S2C4);
  						fillRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, S2C5);
  						fillRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, S2C6);
						if (currentcolor == S2C1) drawRect(0, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S2C2) drawRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S2C3) drawRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S2C4) drawRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S2C5) drawRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S2C6) drawRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, WHITE);
					}
					//Draw in colors for state 3
					else if(color_state==2)
					{
						color_state=3;
						fillRect(0, 0, BOXSIZE, BOXSIZE, S3C1);
  						fillRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, S3C2);
  						fillRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, S3C3);
  						fillRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, S3C4);
  						fillRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, S3C5);
  						fillRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, S3C6);
						if (currentcolor == S3C1) drawRect(0, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S3C2) drawRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S3C3) drawRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S3C4) drawRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S3C5) drawRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S3C6) drawRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, WHITE);
					}
					//Draw in colors for state 0
					else if(color_state==3)
					{
						color_state=0;
						fillRect(0, 0, BOXSIZE, BOXSIZE, S0C1);
  						fillRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, S0C2);
  						fillRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, S0C3);
  						fillRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, S0C4);
  						fillRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, S0C5);
  						fillRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, S0C6);
						if (currentcolor == S0C1) drawRect(0, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S0C2) drawRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S0C3) drawRect(BOXSIZE*2, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S0C4) drawRect(BOXSIZE*3, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S0C5) drawRect(BOXSIZE*4, 0, BOXSIZE, BOXSIZE, WHITE);
          				if (currentcolor == S0C6) drawRect(BOXSIZE*5, 0, BOXSIZE, BOXSIZE, WHITE);
					}
	   			}
			}
			//Draw on screen
			if (((y-pen_size) > BOXSIZE+BOXSIZE) && ((y+pen_size) < TFTHEIGHT)) {
      			fillCircle(x, y, pen_size, currentcolor);
			}
		}
	}
}
